__title__ = "kakao.py"
__author__ = "jhleekr"
__license__ = "MIT"
__copyright__ = "Copyright (C) 2021 jhleekr"
__version__ = "0.2.0"

from .kakao import Client
from .checkreg import check_reg
from .ext import commands


import os
import io
import struct

import bson


class Packet:
    def __init__(self, PacketID=0, StatusCode=0, PacketName="", BodyType=0, Body=b""):
        self.PacketID = PacketID
        self.StatusCode = StatusCode
        self.PacketName = PacketName
        self.BodyType = BodyType
        self.Body = Body

    def toLocoPacket(self):
        f = io.BytesIO()
        f.write(struct.pack("<I", self.PacketID))
        f.write(struct.pack("<H", self.StatusCode))

        if (11 - len(self.PacketName)) < 0:
            raise Exception("invalid packetName")

        f.write(self.PacketName.encode("utf-8"))

        f.write(b"\x00" * (11 - len(self.PacketName)))
        f.write(struct.pack("<b", self.BodyType))
        f.write(struct.pack("<i", len(self.Body)))

        f.write(self.Body)
        return f.getvalue()

    def readLocoPacket(self, packet):
        self.PacketID = struct.unpack("<I", packet[:4])[0]
        self.StatusCode = struct.unpack("<H", packet[4:6])[0]
        self.PacketName = packet[6:17].decode().replace("\0", "")
        self.BodyType = struct.unpack("<b", packet[17:18])[0]
        self.BodySize = struct.unpack("<i", packet[18:22])[0]
        self.Body = packet[22:]

    def toEncryptedLocoPacket(self, crypto):
        iv = os.urandom(16)
        encrypted_packet = crypto.aesEncrypt(self.toLocoPacket(), iv)

        f = io.BytesIO()
        f.write(struct.pack("<I", len(encrypted_packet) + len(iv)))
        f.write(iv)
        f.write(encrypted_packet)

        return f.getvalue()

    def readEncryptedLocoPacket(self, packet, crypto):
        packetLen = struct.unpack(">I", packet[0:4])[0]
        iv = packet[4:20]
        data = packet[20 : packetLen - 16]

        dec = crypto.aesDecrypt(data, iv)

        try:
            self.readLocoPacket(dec)
        except Exception as e:
            print(str(e))

    def toJsonBody(self):
        return bson.decode(self.Body)



import asyncio


class Writer:
    def __init__(self, crypto, StreamWriter, PacketDict):
        self.crypto = crypto
        self.StreamWriter = StreamWriter
        self.PacketID = 0
        self.PacketDict = PacketDict

    def __getPacketID(self):
        self.PacketID += 1
        return self.PacketID

    async def sendPacket(self, packet):
        pid = self.__getPacketID()

        fut = asyncio.get_event_loop().create_future()
        self.PacketDict[pid] = fut

        packet.PacketID = pid
        self.StreamWriter.write(packet.toEncryptedLocoPacket(self.crypto))
        await self.StreamWriter.drain()

        return await fut



from .httpApi import RequestPasscode, RegisterDevice
import subprocess
import sys
import threading
import locale


def check_reg(
    user_id, user_pw, device_name="kakao.py", device_uuid="a2FrYW9weWRldjEyMDI="
):
    try:
        check_registered = input_timer(
            "If you are not registered yet, enter 'y' within 10 seconds: ", 10
        )
    except TimeoutError:
        check_registered = ""

    if check_registered != "y":
        return

    RequestPasscode(user_id, user_pw, device_name, device_uuid)

    print("Please check your phone or computer for the authorization passcode")
    passcode = input("PASSCODE: ")

    RegisterDevice(user_id, user_pw, device_name, device_uuid, passcode)
    # TODO: 성공 여부 체크
    print("Registered!")


def input_timer(prompt, timeout_sec):
    class Local:
        # check if timeout occured
        _timeout_occured = False

        def on_timeout(self, process):
            self._timeout_occured = True
            process.kill()
            # clear stdin buffer (for linux)
            # when some keys hit and timeout occured before enter key press,
            # that input text passed to next input().
            # remove stdin buffer.
            try:
                import termios

                termios.tcflush(sys.stdin, termios.TCIFLUSH)
            except ImportError:
                # windows, just exit
                pass

        def input_timer_main(self, prompt_in, timeout_sec_in):
            # print with no new line
            print(prompt_in, end="")

            # print prompt_in immediately
            sys.stdout.flush()

            # new python input process create.
            # and print it for pass stdout
            cmd = [sys.executable, "-c", "print(input())"]
            with subprocess.Popen(
                cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
            ) as proc:
                timer_proc = threading.Timer(timeout_sec_in, self.on_timeout, [proc])
                try:
                    # timer set
                    timer_proc.start()
                    stdout, stderr = proc.communicate()

                    # get stdout and trim new line character
                    result = stdout.decode(locale.getpreferredencoding()).strip("\r\n")
                finally:
                    # timeout clear
                    timer_proc.cancel()

            # timeout check
            if self._timeout_occured is True:
                # move the cursor to next line
                print("")
                raise TimeoutError
            return result

    t = Local()
    return t.input_timer_main(prompt, timeout_sec)


from setuptools import setup, find_packages

setup(
    name="kakao.py",
    # 프로젝트 명을 입력합니다.
    version="0.2.0",
    # 프로젝트 버전을 입력합니다.
    url="https://github.com/jhleekr/kakao.py",
    # 홈페이지 주소를 입력합니다.
    author="jhleekr",
    # 프로젝트 담당자 혹은 작성자를 입력합니다.
    author_email="jhlee@bfy.kr",
    # 프로젝트 담당자 혹은 작성자의 이메일 주소를 입력합니다.
    description="Very simple kakaotalk LOCO/HTTP API protocol wrapper for python.",
    # 프로젝트에 대한 간단한 설명을 입력합니다.
    packages=find_packages(exclude=["tests", "examples"]),
    # 기본 프로젝트 폴더 외에 추가로 입력할 폴더를 입력합니다.
    long_description=open("README.md").read(),
    # 프로젝트에 대한 설명을 입력합니다. 보통 README.md로 관리합니다.
    long_description_content_type="text/markdown",
    # 마크다운 파일로 description를 지정했다면 text/markdown으로 작성합니다.
    install_requires=["pymongo", "requests", "pycryptodomex"],
    # 설치시 설치할 라이브러리를 지정합니다.
    zip_safe=False,
    classifiers=["License :: OSI Approved :: MIT License"]
    # 기본적으로 LICENSE 파일이 있다고 하더라도 명시적으로 적어 놓는 것이 좋습니다.
)


import json

import requests
import hashlib

from .config import APP_VERSION, OS_VERSION, AGENT, LANG

AUTH_HEADER = f"{AGENT}/{APP_VERSION}/{LANG}"
UTH_USER_AGENT = f"KT/{APP_VERSION} Wd/{OS_VERSION} {LANG}"

LOGIN_URL = "https://ac-sb-talk.kakao.com/win32/account/login.json"
REGISTER_DEVICE_URL = "https://ac-sb-talk.kakao.com/win32/account/register_device.json"
REQUEST_PASSCODE_URL = (
    "https://ac-sb-talk.kakao.com/win32/account/request_passcode.json"
)
MORE_SETTING_URL = (
    f"https://sb-talk.kakao.com/win32/account/more_settings.json?since={0}&lang={LANG}"
)
MEDIA_URL = "https://up-m.talk.kakao.com/upload"


def RequestPasscode(email, password, device_name, device_uuid):
    h = Header(email, device_uuid)
    d = Data(email, password, device_name, device_uuid)
    d["permanent"] = "true"
    d["once"] = "false"
    r = requests.post(REQUEST_PASSCODE_URL, headers=h, data=d)

    return r.content.decode()


def RegisterDevice(email, password, device_name, device_uuid, passcode):
    h = Header(email, device_uuid)
    d = Data(email, password, device_name, device_uuid)
    d["permanent"] = "true"
    d["once"] = "false"
    d["passcode"] = passcode
    r = requests.post(REGISTER_DEVICE_URL, headers=h, data=d)

    return r.content.decode()


def Login(email, password, device_name, device_uuid):
    h = Header(email, device_uuid)
    d = Data(email, password, device_name, device_uuid)
    d["permanent"] = True
    d["forced"] = True
    r = requests.post(LOGIN_URL, headers=h, data=d)

    return r.content.decode()


def Header(email, device_uuid):
    return {
        "Content-Type": "application/x-www-form-urlencoded",
        "A": AUTH_HEADER,
        "X-VC": getXVC(email, device_uuid),
        "User-Agent": UTH_USER_AGENT,
        "Accept": "*/*",
        "Accept-Language": LANG,
    }


def getXVC(email, device_uuid, isFull=False):
    string = f"HEATH|{UTH_USER_AGENT}|DEMIAN|{email}|{device_uuid}".encode("utf-8")
    hash = hashlib.sha512(string).hexdigest()
    if isFull:
        return hash
    return hash[0:16]


def Data(email, password, device_name, device_uuid):
    return {
        "email": email,
        "password": password,
        "device_name": device_name,
        "device_uuid": device_uuid,
        "os_version": OS_VERSION,
    }


def upload(data, dataType, userId):
    r = requests.post(
        MEDIA_URL,
        headers={
            "A": AUTH_HEADER,
        },
        data={
            "attachment_type": dataType,
            "user_id": userId,
        },
        files={
            "attachment": data,
        },
    )
    path = r.content.decode()
    key = path.replace("/talkm", "")
    url = "https://dn-m.talk.kakao.com" + path

    return path, key, url


def postText(chatId, li, text, notice, accessKey, deviceUUID):
    if li == 0:
        url = f"https://talkmoim-api.kakao.com/chats/{chatId}/posts"
    else:
        url = f"https://open.kakao.com/moim/chats/{chatId}/posts?link_id={li}"

    r = requests.post(
        url,
        headers={
            "A": AUTH_HEADER,
            "User-Agent": UTH_USER_AGENT,
            "Authorization": f"{accessKey}-{deviceUUID}",
            "Accept-Language": "ko",
        },
        data={
            "content": json.dumps([{"text": text, "type": "text"}]),
            "object_type": "TEXT",
            "notice": notice,
        },
    )

    print(r.content.decode())
